#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <time.h>
#include <x86intrin.h>


// these should go in a .h but I want to avoid another file for the demo.
double multiply_add(double a, double b, double c);
double multiply_add_2(double a, double b, double c);
double double_array_sum(double* array, uint64_t length);
double circumference(double d);
double circumference2(double d);
int double_cmp(double d1, double d2);
int unsigned_cmp(uint64_t d1, uint64_t d2);

// C equivalents, just to double-check that my examples are correct...
double multiply_add_c(double a, double b, double c) {
    return a + b * c;
}

double double_array_sum_c(double* array, uint64_t length) {
    double total = 0;
    for (uint64_t i = 0; i < length; i++) {
        total += array[i];
    }
    return total;
}

double* random_array(uint64_t length) {
    double* data = (double*)malloc(length * sizeof(double));
    for (uint64_t i = 0; i < length; i++) {
        data[i] = (double)rand() / RAND_MAX - 0.5;
    }
    return data;
}

int double_cmp_c(double a, double b) {
    if ( a < b ) {
        return -1;
    } else if ( a > b ) {
        return 1;
    } else {
        return 0;
    }
}

int main() {
    //srand(time(NULL));

    double result = multiply_add(5, 10, 20);
    printf("%g %g\n", multiply_add_c(5, 10, 20), result);
    printf("%g\n", multiply_add_2(5, 10, 20));
    
    double radius = 10;
    printf("%.14f\n", circumference(radius));
    printf("%.14f\n", circumference2(radius));
    printf("%d\n", double_cmp(2.0, 3.0));
    printf("%d\n", double_cmp(0.0, -3.0));
    printf("%d\n", double_cmp(99.0, 99.0));
    printf("%d\n", unsigned_cmp(1, 2));
    printf("%d\n", unsigned_cmp(8, 3));
    printf("%d\n", unsigned_cmp(9, 9));
        
    uint64_t length = 16;
    double* array = random_array(length);
    double total = double_array_sum(array, length);
    printf("%g %g\n", double_array_sum_c(array, length), total);
    free(array);
    return 0;
}
